<?php
require_once ('boConfig.php');

class boDb {
    function __construct() {
        $username = DB_USER;
        $password = DB_PASS;
        $servername = DB_HOST;
        $dbname = DB_NAME;
        $dbport = DB_PORT;

        // create connection
        $this->conn = mysqli_connect($servername, $username, $password, $dbname, $dbport);

        // check connection
        if (!$this->conn) {
            die("Connection failed: " . mysqli_connect_error() . "\n");
        }
    }

    function __destruct() {
        // close connection
        mysqli_close($this->conn);
        $this->conn = null;
    }

    function generiqSqlRequest($sql) {
        $data = null;
        $result = $this->conn->query($sql);
        if (1 == $result->num_rows) {
            $data = $result->fetch_assoc();
        } else if (1 < $result->num_rows) {
            $data = array();
            // read the data of each row
            while($row = $result->fetch_assoc()) {
                $data[] = $row;
            }
        }
        return($data);
    }

    function getSumOfStrokesPerChannel($minTime = null, $maxTime = null) {
        $sql = "SELECT count(*) as total";
        for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
            $c = CHANNEL_NUMBER[$i];
            $sql .= ", sum(signals.channel$c) as sum$c";
        }
        $sql .= " FROM `strokes`, `signals`";
        $sql .= " WHERE strokes.signal_id = signals.id";
        if (!is_null($minTime)) {
            $sql .= " AND strokes.time_ns_stroke > $minTime";
        }
        if (!is_null($maxTime)) {
            $sql .= " AND strokes.time_ns_stroke <= $maxTime";
        }
        return($this->generiqSqlRequest($sql));
    }

    function getSumOfSignalsPerChannel($minTime = null, $maxTime = null) {
        $sql = "SELECT count(*) as total";
        for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
            $c = CHANNEL_NUMBER[$i];
            $sql .= ", sum(channel$c) as sum$c";
        }
        $sql .= " FROM `signals`";
        $sql .= " WHERE 1";
        if (!is_null($minTime)) {
            $sql .= " AND time_ns >= $minTime";
        }
        if (!is_null($maxTime)) {
            $sql .= " AND time_ns < $maxTime";
        }
        return($this->generiqSqlRequest($sql));
    }

    function getDistanceStroke($channel = null, $minTime = null, $maxTime = null) {
        if (is_null($channel)) {
            $sql = "SELECT min(distance) as min, max(distance) as max FROM `strokes` WHERE 1";
        } else {
            $sql  = "SELECT min(strokes.distance) as min, max(strokes.distance) as max";
            $sql .= " FROM `strokes`, `signals`";
            $sql .= " WHERE strokes.signal_id = signals.id";
            $sql .= " AND signals.channel$channel = 1";
        }
        if (!is_null($minTime)) {
            $sql .= " AND strokes.time_ns_stroke > $minTime";
        }
        if (!is_null($maxTime)) {
            $sql .= " AND strokes.time_ns_stroke <= $maxTime";
        }
        return($this->generiqSqlRequest($sql));
    }

    function getSumOfStrokesPerDistancePerChannel($minDistance, $maxDistance, $minTime = null, $maxTime = null) {
        $sql = "SELECT";
        for($i=0; $i<count(CHANNEL_NUMBER); $i++) {
            $c = CHANNEL_NUMBER[$i];
            if (0 < $i) $sql .= ",";
            $sql .= " sum(signals.channel$c) as sum$c";
        }
        $sql .= " FROM `strokes`, `signals`";
        $sql .= " WHERE strokes.signal_id = signals.id";
        $sql .= " AND strokes.distance >= $minDistance";
        $sql .= " AND strokes.distance < $maxDistance";
        if (!is_null($minTime)) {
            $sql .= " AND strokes.time_ns_stroke > $minTime";
        }
        if (!is_null($maxTime)) {
            $sql .= " AND strokes.time_ns_stroke <= $maxTime";
        }
        return($this->generiqSqlRequest($sql));
    }

    function getStrokePositionForChannels($channelList, $minTime = null, $maxTime = null) {
        $sql = "SELECT strokes.lat_stroke as lat";
        $sql .= ", strokes.lon_stroke as lon";
        for($i=0; $i<count($channelList); $i++) {
            $c = $channelList[$i];
            $sql .= ", signals.channel$c as channel$c";
        }
        $sql .= " FROM `strokes`, `signals`";
        $sql .= " WHERE strokes.signal_id = signals.id";
        if (!is_null($minTime)) {
            $sql .= " AND strokes.time_ns_stroke > $minTime";
        }
        if (!is_null($maxTime)) {
            $sql .= " AND strokes.time_ns_stroke <= $maxTime";
        }
        return($this->generiqSqlRequest($sql));
    }
}
?>